@@ -63,7 +63,7 @@ enum {
     CURSOR_OPT,
 };
 
-static char *short_opts = "xlco:i:w:h:p:b:X:Y:t:";
+static char *short_opts = "xlco:i:w:h:p:b:X:Y:t:r:";
 static struct option long_opts[] = {
     { "extract",		no_argument,    	NULL, 'x' },
     { "list",			no_argument,		NULL, 'l' },
@@ -82,6 +82,7 @@ static struct option long_opts[] = {
     { "alpha-threshold", 	required_argument, 	NULL, 't' },
     { "icon",       	 	no_argument,       	NULL, ICON_OPT	},
     { "cursor",     	 	no_argument,       	NULL, CURSOR_OPT },
+    { "raw", 			required_argument, 	NULL, 'r' },
     { 0, 0, 0, 0 }
 };
 
@@ -179,6 +180,7 @@ display_help(void)
     printf(_("  -Y, --hotspot-y=COORD        match or set cursor hotspot y-coordinate\n"));
     printf(_("  -t, --alpha-threshold=LEVEL  highest level in alpha channel indicating\n"
 	     "                               transparent image portions (default is 127)\n"));
+    printf(_("  -r, --raw=FILENAME           store input file as raw PNG (\"Vista icons\")\n"));
     printf(_("      --icon                   match icons only\n"));
     printf(_("      --cursor                 match cursors only\n"));
     printf(_("  -o, --output=PATH            where to place extracted files\n"));
@@ -212,6 +214,8 @@ main(int argc, char **argv)
     bool create_mode = false;
     FILE *in;
     char *inname;
+    int raw_filec = 0;
+    char** raw_filev = 0;
 
     set_program_name(argv[0]);
 
@@ -282,6 +286,11 @@ main(int argc, char **argv)
 	    if (!parse_int32(optarg, &alpha_threshold) || alpha_threshold < 0)
 		die(_("invalid alpha-threshold value: %s"), optarg);
 	    break;
+	case 'r':
+	    raw_filev = realloc (raw_filev, (raw_filec+1)*sizeof (char*));
+	    raw_filev[raw_filec] = optarg;
+	    raw_filec++;
+	    break;
 	case ICON_OPT:
 	    icon_only = true;
 	    break;
@@ -295,8 +304,11 @@ main(int argc, char **argv)
 
     if (extract_mode + create_mode + list_mode > 1)
 	die(_("multiple commands specified"));
-    if (extract_mode + create_mode + list_mode == 0)
-	die(_("missing argument\nTry `%s --help' for more information."), argv[0]);
+    if (extract_mode + create_mode + list_mode == 0) {
+	warn(_("missing argument"));
+	display_help();
+	exit (1);
+    }
     if (icon_only && cursor_only)
 	die(_("only one of --icon and --cursor may be specified"));
 
@@ -333,9 +345,9 @@ main(int argc, char **argv)
     }
 
     if (create_mode) {
-        if (argc-optind <= 0)
+        if (argc-optind+raw_filec <= 0)
 	    die(_("missing arguments"));
-        if (!create_icon(argc-optind, argv+optind, create_outfile_gen, (icon_only ? true : !cursor_only), hotspot_x, hotspot_y, alpha_threshold, bitdepth))
+        if (!create_icon(argc-optind, argv+optind, raw_filec, raw_filev, create_outfile_gen, (icon_only ? true : !cursor_only), hotspot_x, hotspot_y, alpha_threshold, bitdepth))
             exit(1);
     }
 
